/*
 * Copyright (c) 2004 on behalf of the EU EGEE Project:
 * The European Organization for Nuclear Research (CERN),
 * Istituto Nazionale di Fisica Nucleare (INFN), Italy
 * Datamat Spa, Italy
 * Centre National de la Recherche Scientifique (CNRS), France
 * CS Systeme d'Information (CSSI), France
 * Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 * Universiteit van Amsterdam (UvA), Netherlands
 * University of Helsinki (UH.HIP), Finland
 * University of Bergen (UiB), Norway
 * Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 *
 * Authors: Paolo Badino <Paolo.Badino@cern.ch>
 *
 * Version info: $Id: GSSContext.hxx,v 1.6 2004/08/24 16:06:20 badino Exp $
 *
 * Revision history:
 * $ log$
 *
 */
 
#ifndef GLITE_DATA_IO_GSS_CONTEXT_H_
#define GLITE_DATA_IO_GSS_CONTEXT_H_

#include "globus_gss_assist.h"

namespace log4cpp{
    class Category;
}

/**
 * GSSContext class
 */
class GSSContext{
 
public:
    /**
     * Constructor
     */
    GSSContext();
    
    /**
     * Destructor
     */
    ~GSSContext();
    
    /**
     * Acquire Service Credentials
     */
    int acquireCredentials(const char * service_name);
    
    /**
     * Accept the security context of the incoming connection
     */
    int accept(int s, const char *service_name, const char * buf, int len);
    
    /**
     * Establish a security context with the server
     */
    int establish(int s, char * service_name);
               
    /**
     * Map client to a local user
     */               
    int map(char * &local_name);
       
    /**
     * Wrap a message
     */ 
    int wrap(const char * src_buf, int src_len,char** dest_buf, int *dest_len, 
             int conf_req_flag);
            
    /**
     * Unwrap a message
     */ 
    int unwrap(const char * src_buf, int src_len,char** dest_buf, 
               int *dest_len);

    /**
     * Release a previously allocated buffer
     */ 
    void deleteBuffer(char** dest_buf, int buf_len);

    /**
     * Enable a Service
     */
    void enableService(unsigned int flag);

    /**
     * Check the a given service is available
     */
    int checkService(int flag);

    /**
     * Get Client Name
     */
    const char * getClientName() { return _clientName;}
        
    /**
     * Set Delegated Credentials
     */
    int setDelegatedCredentials(gss_cred_id_t cred);
    
    /**
     * Get Delegated Credentials
     */
    gss_cred_id_t getDelegatedCredentials();
    
    /**
     * Store User's Credentials in a file
     */
    int exportCredentials(const char * filename);
   
    /**
     * Import User's Credentials from a file
     */
    int importCredentials(const char * filename);
   
    /**
     * Export Delegated Credentials to a buffer
     */
    int exportCredentials(char** dest_buf, int* buf_len);
    
    /**
     * Import Credentials from a buffer
     */
    int importCredentials(const char* dest_buf, int buf_len);
    
    /**
     * Initialize GSS API
     */
    static void initGSS();

    /**
     * Finalize GSS API
     */
    static void finiGSS();
        
private:
    /**
     * Send a Token
     */
    int sendToken(int s, gss_buffer_t tok, int timeout);
    /**
     * Receive a Token
     */
    int recvToken(int s, gss_buffer_t tok, int timeout);

    /**
     * Process GSSApi Error
     */
    void processGSSApiError(const char * msg, OM_uint32 maj_stat,
                            OM_uint32 min_stat);
    /**
     * Trace Debug Message only if DEBUG flag is specifyied at compile time
     */                  
    void trace(const char *msg, ...);

    /**
     * Trace Debug Message
     */                  
    void traceDebug(const char *msg, ...);

    /**
     * Trace Info Message
     */                  
    void traceInfo(const char *msg, ...);

    /**
     * Trace Debug Message
     */                  
    void traceWarning(const char *msg, ...);

    /**
     * Trace Error Message
     */                  
    void traceError(const char *msg, ...);
                   
private:

    enum{
        CTX_UNDEF                 = 0x0000,
        CTX_INITIALIZED           = 0x0001,
        CTX_CONTEXT_ESTABLISHED   = 0x0002,
        CTX_CREDENTIALS_LOADED    = 0x0004,
        CTX_CREDENTIALS_DELEGATED = 0x0008
    };

    static bool     _Initialized;

    gss_ctx_id_t    _context;
    gss_cred_id_t   _credentials;
    gss_cred_id_t   _delegCredentials;
    int             _ctxStatus;
    unsigned int    _retFlags;
    unsigned int    _inFlags;
    char *          _clientName;
    bool            _cleanupProxyCred;
    
    /**
     * The Logger
     */
    class log4cpp::Category& _logger;
};

#endif //GLITE_DATA_IO_GSS_CONTEXT_H_

