/*
 * Copyright (c) 2004 on behalf of the EU EGEE Project:
 * The European Organization for Nuclear Research (CERN),
 * Istituto Nazionale di Fisica Nucleare (INFN), Italy
 * Datamat Spa, Italy
 * Centre National de la Recherche Scientifique (CNRS), France
 * CS Systeme d'Information (CSSI), France
 * Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 * Universiteit van Amsterdam (UvA), Netherlands
 * University of Helsinki (UH.HIP), Finland
 * University of Bergen (UiB), Norway
 * Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 *
 * Authors: Paolo Badino <Paolo.Badino@cern.ch>
 *
 * Version info: $Id: QUANTAnet_gss_auth_c.hxx,v 1.5 2004/08/09 09:44:30 badino Exp $
 *
 * Revision history:
 * $ log$
 *
 */
 
#ifndef _QUANTAPLUS_QUANTAPLUS_GSS_AUTH_CLASS
#define _QUANTAPLUS_QUANTAPLUS_GSS_AUTH_CLASS

#include "glite/data/io/quanta/QUANTAnet_auth_strategy_c.hxx"

class GSSContext;

/**
 * QUANTAnet_gss_auth_context_c
 * GSS authorization Context
 */
class QUANTAnet_gss_auth_context_c : public QUANTAnet_auth_context_c{

public:
    /**
     * Destructor
     */
	virtual ~QUANTAnet_gss_auth_context_c();

    /**
     * Return User's Name
     * @return User's Name or 0 if not available
     */
    virtual const char * getName();

    /**
     * Store User's Credentials in a file
     * @param filename [IN] the name of the file where to store the user 
     * credentials
     * @return 0 if success
     */
    virtual int exportCredentials(const char * filename);
   
    /**
     * Export User's Credentials to a buffer. Buffer Should be deallocated by 
     * calling QUANTAnet_auth_context_c::releaseBuffer
     * @param buffer [OUT] the buffer to store the user's credentials
     * @param buf_len [OUT] the buffer length
     * @return 0 if success
     */
    virtual int exportCredentials(char ** buffer, int * buf_len);
   
    /**
     * Check the if a Protection Mode is enabled
     * @param pm [IN] the protection mode to check
     * @return true if the requested protection service is available
     */
    virtual bool checkProtectionMode(ProtectionMode pm);
   
    /**
     * Encrypt Data according the the requested Protection Mode
     * @param src_buffer [IN] the buffer to encrypt
     * @param src_len [IN] the length of the source buffer
     * @param dst_buffer [OUT] the destination buffer. The buffer is allocated 
     *   internally and should be freed using the method @see releaseBuffer
     * @param dst_len [OUT] the length of the destination buffer
     * @return 0 if success
     */
	virtual int encrypt(const char * src_buffer, int src_len, char * &dst_buffer, int &dst_len, ProtectionMode pm = PM_PRIVACY);

    /**
     * Decrypt Data according
     * @param src_buffer [IN] the buffer to decrypt
     * @param src_len [IN] the length of the source buffer
     * @param dst_buffer [OUT] the destination buffer. The buffer is allocated 
     *   internally and should be freed using the method @see releaseBuffer
     * @param dst_len [OUT] the length of the destination buffer
     * @return 0 if success
     */
	virtual int decrypt(const char * src_buffer, int src_len, char * &dst_buffer, int &dst_len);

    /**
     * Release a buffer previosly allocated by @see encrypt or @see descrypt
     * @param buffer [IN] the buffer to release
     */
	virtual void releaseBuffer(char * &buffer, int buffer_len);
   
protected:
    friend class QUANTAnet_gss_server_auth_strategy_c;
    friend class QUANTAnet_gss_client_auth_strategy_c;
    /**
     * Constructor
     * @param ctx [IN] the security Context
     */
    QUANTAnet_gss_auth_context_c(GSSContext * ctx);    
   
private:    
    // Not Implemented
    QUANTAnet_gss_auth_context_c();
    
private:

    /**
     * The GSS Security Context
     */
	GSSContext     * _context;
};

/**
 * QUANTAnet_gss_client_auth_strategy_c
 * GSS authorization Context Client Factory
 */
class QUANTAnet_gss_client_auth_strategy_c: public QUANTAnet_client_auth_strategy_c{

public:
    /**
     * Constructor
     * @param name [IN] the name reported on the server certificate
     */
    QUANTAnet_gss_client_auth_strategy_c(const char * name);   
    
    /**
     * Destructor
     */
    virtual ~QUANTAnet_gss_client_auth_strategy_c();
   
    /**
     * Initialize
     * @return 0 if success
     */
    virtual int init();
    
    /**
     * Enable Encryption Service
     * @return 0 if success
     */
    virtual int enableEncryption();
    
    /**
     * Enable Delegation Service
     * @return 0 if success
     */
    virtual int enableDelegation();
   
    /**
     * Import the Credentials for Delegation from a proxy file
     * @param filename [IN] the file containing the credentials to be used
     * for delegation. This parameters is returned by a call to 
     * QUANTAnet_auth_context_c::exportCredentials
     * @return 0 if success
     */
    virtual int importCredentials(const char * filename);
    
    /**
     * Import the Credentials for Delegation from a buffer created by a call to
     * QUANTAnet_auth_context_c::exportCredentials
     * @param buffer [IN] the buffer containing the credentials to be used
     * for delegation
     * @param buf_len [IN] the length of the buffer
     * @return 0 if success
     */
    virtual int importCredentials(const char * buffer, int buf_len);
    
    /**
     * Establish the Authorization Context 
     * @param sock_handle [IN] handle of the socket used to contact the peer
     * @return the established Security Context
     */
    virtual QUANTAnet_auth_context_c * establish(int sock_handle);
    
private:

    /**
     * Name Reported on the Server Certificate
     */
    char * _name;

    /**
     * Proxy Credentials To be Used for delegation
     */    
    const char * _delegCredProxy;
    
    /**
     * Proxy Credentials To be Used for delegation
     */    
    char * _delegCredBuf;

    /**
     * Length of Credentials Buffer
     */    
    int _delegCredBufLen;
    
    /**
     * Services to be enabled
     */
    unsigned int _requestedServices;
};

/**
 * QUANTAnet_gss_server_auth_strategy_c
 * GSS authorization Context Server Factory
 */
class QUANTAnet_gss_server_auth_strategy_c: public QUANTAnet_server_auth_strategy_c{

public:
    /**
     * Constructor
     * @param name [IN] the name reported on the server certificate
     */
    QUANTAnet_gss_server_auth_strategy_c(const char * name);
    /**
     * Destructor
     */
    virtual ~QUANTAnet_gss_server_auth_strategy_c();
 
    /**
     * Initialize
     * @return 0 if success
     */
    virtual int init();

    /**
     * Accept a Security Context 
     * @param sock_handle [IN] handle of the socket used to contact the peer
     * @return the established Security Context
     */
    virtual QUANTAnet_auth_context_c * accept(int sock_handle);

    /**
     * Authorize an established Security Context 
     * @param ctx [IN] the security context to be authorized
     * @return 0 if success
     */
    virtual int authorize(QUANTAnet_auth_context_c * ctx);

private:
    /**
     * Name Reported on the Server Certificate
     */
    char * _name;
    
};

#endif
